"""
@S <a> <b> <c> <d> <e> <f> <g> <h> <i> <j> <k> <l>
Outputs information about the mp3 file after loading.
<a>: version of the mp3 file. Currently always 1.0 with madlib, but don't 
     depend on it, particularly if you intend portability to mpg123 as well.
     Float/string.
<b>: layer: 1, 2, or 3. Integer.
<c>: Samplerate. Integer.
<d>: Mode string. String.
<e>: Mode extension. Integer.
<f>: Bytes per frame (estimate, particularly if the stream is VBR). Integer.
<g>: Number of channels (1 or 2, usually). Integer.
<h>: Is stream copyrighted? (1 or 0). Integer.
<i>: Is stream CRC protected? (1 or 0). Integer.
<j>: Emphasis. Integer.
<k>: Bitrate, in kbps. (i.e., 128.) Integer.
<l>: Extension. Integer.
"""

import popen2, os, config

class mpg123:
	"Controller for remote MPG"
	Fields = '''
	version layer sampleRate modeString modeExtension bytesPerFrame
	channels copyrighted protected emphasis bitrate extension
	'''.strip().split()
	FrameFields = 'currentFrame framesRemaining currentTime timeRemaining'.split()

	def __init__(self, onStop):
		self.onStop = onStop
		self.child = popen2.Popen4(config.mpg123_command + ' ' + config.mpg123_flags + ' -R foo')
		for x in self.Fields:
			setattr(self,x,'')
		for x in self.FrameFields:
			setattr(self,x,0)
		self.buf = ''

	def fileno(self):
		return self.child.fromchild.fileno()

	def update(self):
		self.buf += os.read(self.child.fromchild.fileno(),4096)
		while 1:
			nl = self.buf.find('\n')
			if nl == -1:
				break
			line = self.buf[:nl]
			self.buf = self.buf[nl+1:]
        
			what, args = line.split(None,1)
			if what == '@S':
				for field, value in zip(self.Fields, args.split()):
					setattr(self, field, value)
			elif what == '@F':
				for field, value in zip(self.FrameFields, args.split()):
					setattr(self, field, float(value))
			elif what == '@P':
				x = int(args)
				if x == 0:
					self.onStop()

	def cmd(self,cmd):
		#sys.stderr.write('Sending command: %s\n' % cmd)
		self.child.tochild.write('%s\n' % cmd)
		self.child.tochild.flush()

	def play(self, song):
		self.cmd('LOAD %s' % song)

	def pause(self):
		self.cmd('PAUSE')

	def jump(self, offset):
		self.cmd('JUMP %+d' % offset)

	def pause(self):
		self.cmd('PAUSE')
	
	def stop(self):
		self.cmd('STOP')

	def quit(self):
		self.cmd('QUIT')

	def getTime(self):
		return (self.currentTime, self.timeRemaining)
	
	def __del__(self):
		self.quit()
