#!/usr/bin/python

# pylayer.py

# this is the control class for pylayer
# it is also the second revision of the pylayer module

import files, curses, curses.ascii, os.path, mpg123, select, string
from config import *

class pylayer:
	def __init__(self):
		pass
	def initialise(self):
		# initialize playlist
		self.playlist	= files.files(mp3_location)

		# initialize curses
		self.stdscr		= curses.initscr()
		curses.noecho()
		try: curses.curs_set(0)
		except: pass	# this option doesn't work on all terminals
		self.create_windows()
		self.stdscr.timeout(500)
		self.stdscr.keypad(1)

		# initialize globals
		self.playqueue			= []
		self.mp3_namer			= self.playlist.renameMP3s
		self.current_song		= ''
		self.player_status		= 0
		self.lock_interface		= False
		self.track_total		= 0
		self.old_elapsed		= 0
		
		# text scrolling variables
		self.scroll_title		= 0
		self.scroll_direction	= 1
		self.scroll_delayed		= False

		#initialize player
		self.player			= mpg123.mpg123(self.need_awaiting)
		
		# register fd's for reading
		self.poll			= select.poll()
		self.poll.register(self.player)

		# display interfaces
		self.redraw_screen()
		self.stdscr.refresh()

		# do we do mp3s on startup?
		if play_on_start:
			self.player_status = 3
			self.awaiting()
	
	def create_windows(self):
		self.term_height, self.term_width = self.stdscr.getmaxyx()
	
		self.win_tl		= self.stdscr.subwin(12, 40, 0, 0)
		self.win_tr		= self.stdscr.subwin(12, self.term_width - 40, 0, 40)
		self.win_b		= self.stdscr.subwin(self.term_height - 12, self.term_width, 12, 0)

	def redraw_screen(self):
		self.win_tl.clear()
		self.win_tl.box()
		self.draw_win_tl()
		
		self.win_tr.clear()
		self.win_tr.box()
		self.draw_win_tr()
		
		self.win_b.clear()
		self.win_b.box()
		self.draw_win_b()
		
	def __del__(self):
		# end the curses environment
		self.stdscr.keypad(0)
		self.stdscr.nodelay(0)
		self.player.quit()
		try: curses.curs_set(1)
		except: pass # doesn't work on all terminals
		curses.echo()
		curses.endwin()
	
	def format_current_song(self):
		self.current_song_f = self.mp3_namer.sub('\\1', os.path.basename(self.current_song))

	def draw_win_tl(self):
		self.win_tl.addstr(0, 2, ' p(y)layer - MP3 jukebox ')
		# and some instructions
		if self.playlist.canGetLast(): self.win_tl.addstr(2, 3, 'Z - Last Song')
		else: self.win_tl.addstr(2, 3, ' ' * 14)
		self.win_tl.addstr(3, 3, 'X - Play')
		self.win_tl.addstr(4, 3, 'C - Pause')
		self.win_tl.addstr(5, 3, 'V - Stop')
		self.win_tl.addstr(6, 3, 'B - Next Song')
		self.win_tl.addstr(8, 3, 'J - Find Song')
		self.win_tl.addstr(9, 3, 'Q - Quit')
	
	def draw_win_tr(self):
		self.win_tr.addstr(0, 2, " Now Playing ")

		statuses	= [ 'Stopped', 'Paused ', 'Playing', 'ERROR' ]
		
		self.win_tr.addstr(2, 3, statuses[self.player_status])
		if self.player_status != 0:
			(elapsed, remaining)	= self.player.getTime()
			# calculate song rotation
			if len(self.current_song_f) > self.term_width - 45:
				e = int(elapsed)
				if e - 60 * (e / 60) != self.old_elapsed:
					if not self.scroll_delayed: self.scroll_title += self.scroll_direction
					if len(self.current_song_f) - self.scroll_title - self.term_width + 45 < 1 or self.scroll_title == 0:
						# we want a two second delay
						if self.scroll_delayed:
							self.scroll_direction *= -1
							self.scroll_delayed = False;
						else:
							self.scroll_delayed = True;
				title	= self.current_song_f[self.scroll_title:self.scroll_title + self.term_width - 45]
			else:
				title = self.current_song_f
			self.win_tr.addstr(3, 3, title)
		else:
			elapsed, remaining		= 0, 0
			self.win_tr.addstr(3, 3, ' ' * (self.term_width - 45))

		e = int(elapsed)
		remaining = int(remaining)
		
		if e + remaining > self.track_total: self.track_total = e + remaining
		
		remaining = self.track_total - e
		
		elap = string.zfill(e / 60, 2) + ':' + string.zfill(e - 60 * (e / 60), 2)
		rema = string.zfill(remaining / 60, 2) + ':' + string.zfill(remaining - 60 * (remaining / 60), 2)
		
		self.old_elapsed = e - 60 * (elapsed / 60)
		
		self.win_tr.addstr(5, 3, 'Elapsed:   ' + elap)
		self.win_tr.addstr(6, 3, 'Remaining: ' + rema)
		
		try: percent = elapsed / self.track_total
		except: percent = 0

		percent_s	= string.zfill(int(percent * 100), 2)
		percent_i	= int(percent * (self.term_width - 55))
		self.win_tr.addstr(8, 3, percent_s + '%' + ' ' * (4 - len(percent_s)) + '['
			+ '*' * percent_i + ' ' * (self.term_width - 55 - percent_i) + ']')
	
	def draw_win_b(self):
		self.win_b.addstr(0, 2, ' Coming Up ')

		lineno		= 2
		linesavail	= self.term_height - 15
		if len(self.playqueue) > linesavail:
			linesavail -= 1
			self.win_b.addstr(lineno + linesavail, 10, 'Plus ' + str(len(self.playqueue) - linesavail) + ' more songs...')
		
		for song in self.playqueue[:linesavail]:
			self.win_b.addstr(lineno, 3, self.mp3_namer.sub('\\1', os.path.basename(song))[:self.term_width - 2])
			lineno += 1
			
	def need_awaiting(self):
		# this is a wrapper class for when the mp3 runs out
		if self.player_status != 0: self.player_status = 3
		self.awaiting()

	def awaiting(self):
		# we need a new song, now!
		if self.player_status != 0:
			if self.player_status == 3: self.playlist.addPast(self.current_song)
			# grab a song
			if len(self.playqueue) > 0:
				self.current_song	= self.playqueue.pop(0)
			else:
				self.current_song	= self.playlist.getRandom()
			# play it
			self.player.play(self.current_song)
			self.player_status		= 2
			self.scroll_title		= 0
			self.scroll_direction	= 1
			self.track_total		= 0
			
			# handle the interfaces
			self.format_current_song()
			if not self.lock_interface:
				self.redraw_screen()
				self.draw_win_tl()
				self.draw_win_tr()
				self.draw_win_b()
				self.win_tl.refresh()
				self.win_tr.refresh()
				self.win_b.refresh()
	
	def paused(self):
		if self.player_status == 1: self.player_status		= 2
		elif self.player_status == 2: self.player_status	= 1
		if not self.lock_interface:
			self.draw_win_tr()
			self.win_tr.refresh()
	
	def find_input(self):
		self.searchstring	= ''
		self.pointer	= 0
		self.f_start	= 0
		self.f_end		= self.term_height - 9
		width			= self.term_width - 34
		self.tainted	= False
		
		while True:
			self.find.addstr(2, 2, self.searchstring[-width:].strip() + ' ' * (width - len(self.searchstring)), curses.A_REVERSE)
			self.find.refresh()
	
			c		= self.stdscr.getch()

			if c == -1:
				# this it the timeout case, handle anything that isn't immediate
				if self.poll.poll()[0][1] & select.POLLIN: self.player.update()				
				if self.tainted: self.find_update()
			elif c == curses.KEY_RESIZE:
				# resize event
				width = self.term_width - 34
				self.create_windows()
				self.redraw_screen()
				self.draw_win_tl()
				self.draw_win_tr()
				self.draw_win_b()
				self.create_find()
				self.stdscr.refresh()
				self.tainted = True
			elif c == curses.ascii.LF:
				# ENTER
				# force the update, then break
				self.find_update()
				break
			elif c == ord(curses.ascii.ctrl('U')):
				# ^U
				self.playlist.resetSearch()
				self.searchstring = ''
				self.tainted	= True
			elif c == curses.KEY_DOWN:
				# DOWN ARROW
				if self.pointer < len(self.playlist.getSearch()) - 1: self.pointer += 1
				self.find_update()
			elif c == curses.KEY_UP:
				# UP ARROW
				if self.pointer > 0: self.pointer -= 1
				self.find_update()
			elif c == curses.ascii.ESC:
				# ESCAPE
				self.searchstring = ''
				break
			elif c == curses.KEY_BACKSPACE:
				# BACKSPACE
				self.playlist.delSearchCharacter(1)
				self.searchstring = self.searchstring[:-1]
				self.tainted	= True
			elif curses.ascii.isalnum(c) or curses.ascii.isspace(c) and c != 9:
				self.searchstring += chr(c).lower()
				self.playlist.addSearchCharacter(chr(c))
				self.tainted	= True

		# we've broken, now select the song
		# we need to ensure we can't grab a non existant song
		try: match = self.playlist.getSearch()[self.pointer]
		except: match = None
		if self.searchstring != '' and match != None: self.playqueue.append(self.playlist.getSearch()[self.pointer])

	def find_update(self):
		width				= self.term_width - 35
		nolines				= self.term_height - 9
		# do a search
		if self.tainted:
			self.playlist.doSearch()
			self.pointer	= 0
			self.f_start	= 0
			self.f_end		= self.term_height - 9
			self.tainted	= False
		# manipulate the view window for scrolling
		if self.pointer < self.f_start:
			self.f_start	= self.pointer
			self.f_end		= self.f_start + nolines
		elif self.pointer > self.f_end - 1:
			self.f_end		= self.pointer + 1
			self.f_start	= self.f_end - nolines
			if self.f_start < 0: self.f_start = 0 
		else: pass
		# draw the search
		if self.searchstring.strip() != '': matches = self.playlist.getSearch()[self.f_start:self.f_end]
		else: matches = []
		lineno				= 4
		lineno_o			= lineno
		for match in matches:
			trackname		= self.mp3_namer.sub('\\1', os.path.basename(match))
			if self.f_start > 0 and lineno == lineno_o: self.find.addch(lineno, 1, curses.ACS_UARROW)
			elif lineno == lineno_o: self.find.addstr(lineno, 1, ' ')
			elif self.f_end < len(self.playlist.getSearch()) and lineno - lineno_o == nolines - 1: self.find.addch(lineno, 1, curses.ACS_DARROW)
			elif lineno - lineno_o == nolines - 1: self.find.addstr(lineno, 1, ' ')
			if lineno - lineno_o + self.f_start == self.pointer: attrib = curses.A_REVERSE
			else: attrib = curses.A_NORMAL
			self.find.addstr(lineno, 3, trackname[:width - 1] + ' ' * (width - len(trackname) - 1), attrib)
			lineno += 1
		for line in range(lineno, nolines + lineno_o):
			self.find.addstr(line, 1, ' ' * (width + 1))
	
	def create_find(self):
		self.find	= self.stdscr.subwin(self.term_height - 4, self.term_width - 30, 2, 15)
		self.find.clear()
		self.find.box()
		self.find.addstr(0, 2, ' Find Song ')
	
	def draw_find(self):
		self.playlist.resetSearch()
		self.create_find()
		self.find_input()
		del(self.find)
	
	def input(self):
		while True:
			# lets try for some input
			char = self.stdscr.getch()
			
			try: c = chr(char).lower()
			except: c = None
	
			if char == -1:
				# this is the timeout case, handle anything that isn't immediate
				event = self.poll.poll()[0][1]
				if event & select.POLLIN: self.player.update()
				elif event & select.POLLHUP:
					# oops, mpg123, died, lets reinitialize it, and call awaiting()
					# or something like that
					self.poll.unregister(self.player)
					# try to clean up, it may be needed
					del(self.player)
					# reinitialize
					self.player = mpg123.mpg123(self.awaiting)
					self.awaiting()
					# reregister with poll()
					self.poll.register(self.player)
				if not self.lock_interface:
					self.draw_win_tr()
					self.win_tr.refresh()
			elif char == curses.KEY_RESIZE and not self.lock_interface:
				# resize the window
				self.create_windows()
				self.redraw_screen()
				self.draw_win_tl()
				self.draw_win_tr()
				self.draw_win_b()
				self.stdscr.refresh()
			elif c == 'z' and self.playlist.canGetLast():
				self.playqueue.insert(0,self.playlist.getLast())
				self.player_status = 4
				self.awaiting()
			elif c == 'x':
				if self.player_status == 1:
					self.player.pause()
					self.paused()
				elif self.player_status != 2:
					self.player_status = 2
					self.awaiting()
			elif c == 'c':
				self.player.pause()
				self.paused()
			elif c == 'v':
				self.player.stop()
				self.player_status = 0
				if not self.lock_interface:
					self.draw_win_tr()
					self.win_tr.refresh()
			elif c == 'b' and (self.player_status == 1 or self.player_status == 2):
				# this is now the same call that it makes automatically
				self.need_awaiting()
			elif c == 'j':
				self.lock_interface	= True
				self.draw_find()
				self.lock_interface	= False
				# display interfaces
				self.redraw_screen()
				self.draw_win_tl()
				self.draw_win_tr()
				self.draw_win_b()
				self.stdscr.refresh()										
			elif c == 'q' or char == curses.ascii.ESC:
				break
